/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <stdlib.h>
#include "TFDialogRoughSmooth.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "GuiDialogOAC.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogRoughSmooth::TFDialogRoughSmooth (HeightField *HF, HeightFieldDraw *HFD)
	        : TFPreviewDialog (HF, HFD, "Terraform Roughen/Smooth Dialog", 
					"Roughen/Smooth Parameters", 6, 4),
		  d_vBoxList (TRUE, 5),
		  d_hBoxMode (TRUE, 5),
		  d_hBoxRSFactor (TRUE, 5),
		  d_hBoxBigGrid (TRUE, 5),
		  d_adjRSFactor (0.5, 0, 1, 0.01),
		  d_hsRSFactor (d_adjRSFactor),
		  d_lblMode ("Mode"),
		  d_lblRSFactor ("Adjustment Factor"),
		  d_lblBigGrid ("Big Sampling Grid"),
		  d_optionsFrame ("Roughen/Smooth Options")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogRoughSmooth\n");

	d_rbModeR = d_rbModeS = NULL;
	buildDialogWindow ();
	this->setHFobjs (HF, HFD);
	this->iterateEvents ();
	updatePreviewCallback ();

	connect_to_method (d_rbModeR->clicked, this, &TFDialogRoughSmooth::updatePreviewCallback);
	connect_to_method (d_rbModeS->clicked, this, &TFDialogRoughSmooth::updatePreviewCallback);
	connect_to_method (d_adjRSFactor.value_changed, this, &TFDialogRoughSmooth::updatePreviewCallback);
	connect_to_method (d_btnBigGrid.clicked, this, &TFDialogRoughSmooth::updatePreviewCallback);
}


/*
 *  destructor: clean up 
 */
TFDialogRoughSmooth::~TFDialogRoughSmooth ()
{
	if (d_rbModeR)
		delete d_rbModeR;
	if (d_rbModeS)
		delete d_rbModeS;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogRoughSmooth\n");
}


/*
 *  insertOptions: add the actual dialog options 
 */
void TFDialogRoughSmooth::insertOptions ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogRoughSmooth called without Frame label",
		"TFDialogRoughSmooth::insertOptions");

	d_optionsFrame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_optionsFrame.set_border_width (5);

	d_rbModeR = new Gtk_RadioButton ((GSList *)NULL, "Roughen");
	d_rbModeR->set_active (FALSE);
	d_hBoxMode.pack_start (*d_rbModeR, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_rbModeR->show ();
	d_group = d_rbModeR->group ();
	d_rbModeS = new Gtk_RadioButton (d_group, "Smooth");
	d_rbModeS->set_active (TRUE);
	d_hBoxMode.pack_end (*d_rbModeS, TRUE, TRUE, 2);
	d_rbModeS->show ();
	d_vBoxList.pack_start (d_hBoxMode, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxMode.show ();

	d_hBoxRSFactor.pack_start (d_lblRSFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblRSFactor.show ();
	d_hsRSFactor.set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsRSFactor.set_digits (2);
	d_hsRSFactor.set_draw_value (TRUE);
	d_hBoxRSFactor.pack_end (d_hsRSFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsRSFactor.show ();
	d_vBoxList.pack_start (d_hBoxRSFactor, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxRSFactor.show ();

	d_hBoxBigGrid.pack_start (d_lblBigGrid, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblBigGrid.show ();
	d_btnBigGrid.set_active (FALSE);
	d_hBoxBigGrid.pack_end (d_btnBigGrid, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_btnBigGrid.show ();
	d_vBoxList.pack_start (d_hBoxBigGrid, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxBigGrid.show ();

	d_optionsFrame.add (d_vBoxList);	
	this->d_table->attach (d_optionsFrame, 3, 4, 1, 3);
	d_vBoxList.show ();
	d_optionsFrame.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}



void TFDialogRoughSmooth::buildDialogWindow ()
{
	this->set_usize (450, 225);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->insertPreview ();
	insertOptions ();
	this->fillActionArea ();
	
	this->show ();
}


/*
 *  buttonCallbackApply: transform the height field
 */
void TFDialogRoughSmooth::buttonCallbackApply ()
{
	bool		roughen = d_rbModeR->get_active (),
			bigGrid = d_btnBigGrid.get_active (); 
	float		value = d_adjRSFactor.get_value (); 

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogRoughSmooth::buttonCallbackApply");

	d_HFO->roughSmooth (roughen, bigGrid, value);
	d_HFD->draw ();

	// only do this if window stays open 
	if (d_applyHit)
		{
		this->setHFobjs (d_HF, d_HFD);
		updatePreviewCallback ();
		}

	this->d_applyHit = TRUE;
}


/*
 *  updatePreviewCallback: update the preview after a widget has been changed
 */
void TFDialogRoughSmooth::updatePreviewCallback ()
{
	if (!d_cbUsePreview.get_active())
		return;

	bool		roughen = d_rbModeR->get_active (),
			bigGrid = d_btnBigGrid.get_active (); 
	float		value = d_adjRSFactor.get_value (); 

	SanityCheck::bailout ((!d_HFPreview), "d_HFPreview==NULL", "TFDialogRoughSmooth::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFOPreview), "d_HFOPreview==NULL", "TFDialogRoughSmooth::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFDPreview), "d_HFDPreview==NULL", "TFDialogRoughSmooth::updatePreviewCallback");
	this->previewUpToDate ();
	d_HFPreview->restore ();
	d_HFOPreview->roughSmooth (roughen, bigGrid, value);
	d_HFDPreview->setColormap (d_HFD->getColormap());
	d_HFDPreview->draw ();
}
